package MT::Plugin::SKR::AlertLongSilence;

use strict;
use MT 4;
use MT::Blog;
use MT::Permission;
use MT::Author;
use MT::Entry;
use MT::Template;
use MT::Mail;
use Data::Dumper;#DEBUG

use vars qw( $MYNAME $VERSION $SCHEMA_VERSION );
$MYNAME = 'AlertLongSilence';
$VERSION = '0.02';

$SCHEMA_VERSION = '0.0105';

use base qw( MT::Plugin );
my $plugin = __PACKAGE__->new({
        name => $MYNAME,
        id => lc $MYNAME,
        key => lc $MYNAME,
        version => $VERSION,
        schema_version => $SCHEMA_VERSION,
        author_name => 'SKYARC System Co.,Ltd.',
        author_link => 'http://www.skyarc.co.jp/',
#        doc_link => '',
        description => <<HTMLHEREDOC,
<__trans phrase="Alert to blog when the author is in a long silence.">
HTMLHEREDOC
        blog_config_template => 'config.tmpl',
        settings => new MT::PluginSettings ([
            [ 'timespan', { Default => 0, Scope => 'blog' } ],
        ]),
});
MT->add_plugin ($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        default_templates => {
            base_path => 'templates/',
            'global:email' => {
                long_silence_notify => {
                    label => 'Long Silence Notify',
                },
            },
        },
        tasks => {
            $MYNAME => {
                name        => $MYNAME,
                frequency   => 1,#60 * 60 * 24,
                code        => \&_hdlr_long_silence_notify,
            },
        },
    });
}

### Periodic tasks
sub _hdlr_long_silence_notify {
    my @t1 = gmtime (time);
    my $now_ts = sprintf '%04d%02d%02d%02d%02d%02d', $t1[5]+1900, $t1[4]+1, @t1[3,2,1,0];

    my %author_id = ();
    my @blogs = MT::Blog->load ();
    foreach my $blog (@blogs) {
        my $timespan = $plugin->get_config_value ('timespan', 'blog:'. $blog->id)
            or next; # not yet configured
        my @t2 = gmtime (time - $timespan * 60 * 60 * 24);
        my $from_ts = sprintf '%04d%02d%02d%02d%02d%02d', $t2[5]+1900, $t2[4]+1, @t2[3,2,1,0];
        my @perms = MT::Permission->load({ blog_id => $blog->id })
            or next;
        foreach (@perms) {
            next unless $_->can_create_post || $_->can_edit_entry;
            my $author = MT::Author->load({ id => $_->author_id, status => MT::Author::ACTIVE() })
                or next;
            next if $author_id{$author->id}++;
            MT::Entry->load({
                author_id => $author->id, authored_on => [ $from_ts, $now_ts ],
            }, {
                range => { authored_on => 1 },
            }) and next;

            # Send notification mail
            _send_notify_mail ($blog, $author);
        }
    }
}

### Send notification mail
sub _send_notify_mail {
    my ($blog, $author) = @_;

    my $tmpl = MT::Template->load({ blog_id => 0, identifier => 'long_silence_notify' })
        or return;
    my $ctx = $tmpl->context;
    $ctx->stash ('blog', $blog);
    $ctx->stash ('author', $author);
    my $mail_body = $tmpl->output
        or return;
    $mail_body = Encode::is_utf8 ($mail_body) # for MT5
        ? Encode::decode_utf8 ($mail_body)
        : $mail_body;
    # Use first line as a mail title
    my ($mail_title) = $mail_body =~ /^(.+)/;

    my $cfg = MT->instance->config;
    my %head = (
        $cfg->EmailAddressMain ? (From => $cfg->EmailAddressMain) : (),
        To => $author->email,
        Subject => $mail_title,
    );
    MT::Mail->send (\%head, $mail_body)
        or die MT::Mail->errstr;
}

1;
__END__